@@ -5,11 +5,11 @@ from openapi_core.spec import Spec
 from openapi_core.validation.request import openapi_request_validator
 from openapi_core.validation.request.datatypes import RequestValidationResult
 from openapi_core.validation.request.protocols import Request
-from openapi_core.validation.request.validators import RequestValidator
+from openapi_core.validation.request.protocols import RequestValidator
 from openapi_core.validation.response import openapi_response_validator
 from openapi_core.validation.response.datatypes import ResponseValidationResult
 from openapi_core.validation.response.protocols import Response
-from openapi_core.validation.response.validators import ResponseValidator
+from openapi_core.validation.response.protocols import ResponseValidator
 
 
 def validate_request(
